@@ -24,7 +24,7 @@
     else { // rows are toplevel rows, this is the initial call
       var anchor = window.location.hash.substr(1);
       if (anchor)
-        autoexpand = anchor.split("/");
+        autoexpand = decodeURI(anchor).split("/");
     }
 
     var autoexpand_expander = null;
@@ -74,10 +74,16 @@
 
     // update location, unless autoexpand in progress
     var a = expander.next("a");
-    if ( !autoexpand )
-      window.location.hash = a.attr("href")
-        .substr(window.location.pathname.length+1)
-        .replace(/([^?]*)(\?.*)?$/, '$1');    
+    if ( !autoexpand ) {
+      var pathname = window.location.pathname;
+      // strip scheme and hostname, href is absolute with IE 6 and 7.
+      var entry_href = a.attr("href").replace(/^[^:]+:\/\/[^\/]+/, '');
+      // normalize (PATH_INFO possibly has squashed "/")
+      pathname = pathname.replace(/\/+/g, '/').replace(/\/$/, '');
+      entry_href = entry_href.replace(/\/+/g, '/');
+      window.location.hash = entry_href.substr(pathname.length + 1)
+        .replace(/([^?]*)(\?.*)?$/, '$1');
+    }
 
     // update sort links in column headers
     tr.parents("table:first").find("thead tr:first").find("a").each(function(){
@@ -103,7 +109,7 @@
       tr.addClass("expanded");
       // insert "Loading ..." row
       var loading_row = $($.htmlFormat(
-        '<tr>'+
+        '<tr>' +
         ' <td class="$td_class" colspan="$colspan" ' +
         '     style="padding-left: ${depth}px">' +
         '  <span class="loading">${loading}</span>' +
@@ -112,7 +118,7 @@
         td_class: td_class, 
         colspan: tr.children("td").length, 
         depth: depth, 
-        loading: babel.format(_("Loading %(entry)s..."), {entry: a.text()}),
+        loading: babel.format(_("Loading %(entry)s..."), {entry: a.text()})
       }));
       tr.after(loading_row);
   
