@@ -17,7 +17,7 @@
 import pkg_resources
 import re
 
-from genshi.builder import tag, Element
+from genshi.builder import tag
 
 from trac.config import IntOption, ListOption
 from trac.core import *
@@ -25,7 +25,9 @@ from trac.mimeview import Context
 from trac.perm import IPermissionRequestor
 from trac.search.api import ISearchSource
 from trac.util.datefmt import format_datetime
+from trac.util.html import find_element
 from trac.util.presentation import Paginator
+from trac.util.text import quote_query_string
 from trac.util.translation import _
 from trac.web import IRequestHandler
 from trac.web.chrome import add_link, add_stylesheet, add_warning, \
@@ -47,10 +49,16 @@ class SearchModule(Component):
         """Minimum length of query string allowed when performing a search.""")
 
     default_disabled_filters = ListOption('search', 'default_disabled_filters',
-        doc="""Specifies which search filters should be disabled by default
-               on the search page. This will also restrict the filters for the
-               quick search function. Search filters can still be manually
-               enabled by the user on the search page. (since 0.12)""")
+        doc="""Specifies which search filters should be disabled by
+               default on the search page. This will also restrict the
+               filters for the quick search function. The filter names
+               defined by default components are: `wiki`, `ticket`,
+               `milestone` and `changeset`.  For plugins, look for
+               their implementation of the ISearchSource interface, in
+               the `get_search_filters()` method, the first member of
+               returned tuple. Once disabled, search filters can still
+               be manually enabled by the user on the search page.
+               (since 0.12)""")
 
     # INavigationContributor methods
 
@@ -83,6 +91,7 @@ class SearchModule(Component):
         available_filters = []
         for source in self.search_sources:
             available_filters.extend(source.get_search_filters(req) or [])
+        available_filters.sort(key=lambda f: f[1].lower())
         
         filters = self._get_selected_filters(req, available_filters)
         data = self._prepare_data(req, query, available_filters, filters)
@@ -118,10 +127,13 @@ class SearchModule(Component):
 
     def _format_link(self, formatter, ns, target, label):
         path, query, fragment = formatter.split_link(target)
-        if query:
-            href = formatter.href.search() + query.replace(' ', '+')
-        else:
+        if path:
             href = formatter.href.search(q=path)
+            if query:
+                href += '&' + quote_query_string(query[1:])
+        else:
+            href = formatter.href.search() + quote_query_string(query)
+        href += fragment
         return tag.a(label, class_='search', href=href)
 
     # IRequestHandler helper methods
@@ -152,9 +164,9 @@ class SearchModule(Component):
             name = kwd
             description = _('Browse repository path %(path)s', path=kwd)
         else:
-            link = extract_link(self.env, Context.from_request(req, 'search'),
-                                kwd)
-            if isinstance(link, Element):
+            context = Context.from_request(req, 'search')
+            link = find_element(extract_link(self.env, context, kwd), 'href')
+            if link is not None:
                 quickjump_href = link.attrib.get('href')
                 name = link.children
                 description = link.attrib.get('title', '')
