@@ -139,16 +139,15 @@ class TestSetup(unittest.TestSuite):
         self.tearDown()
         return result
 
-    def _wrapped_run(self, result):
+    def _wrapped_run(self, *args, **kwargs):
         "Python 2.7 / unittest2 compatibility - there must be a better way..."
         self.setUp()
         if hasattr(self, 'fixture'):
             for test in self._tests:
                 if hasattr(test, 'setFixture'):
                     test.setFixture(self.fixture)
-        unittest.TestSuite._wrapped_run(self, result)
+        unittest.TestSuite._wrapped_run(self, *args, **kwargs)
         self.tearDown()
-        return result
 
 class TestCaseSetup(unittest.TestCase):
     def setFixture(self, fixture):
@@ -178,9 +177,16 @@ def reset_postgres_db(db, db_prop):
     if dbname:
         cursor = db.cursor()
         # reset sequences
-        cursor.execute('SELECT sequence_name '
-                       'FROM information_schema.sequences '
-                       'WHERE sequence_schema=%s', (dbname,))
+        # information_schema.sequences view is available in PostgreSQL 8.2+
+        # however Trac supports PostgreSQL 8.0+, uses pg_get_serial_sequence()
+        cursor.execute("""
+            SELECT sequence_name FROM (
+                SELECT pg_get_serial_sequence(%s||table_name, column_name)
+                       AS sequence_name
+                FROM information_schema.columns
+                WHERE table_schema=%s) AS tab
+            WHERE sequence_name IS NOT NULL""",
+            (dbname + '.', dbname))
         for seq in cursor.fetchall():
             cursor.execute('ALTER SEQUENCE %s RESTART WITH 1' % seq)
         # clear tables
@@ -239,9 +245,10 @@ class EnvironmentStub(Environment):
     def __init__(self, default_data=False, enable=None):
         """Construct a new Environment stub object.
 
-        default_data: If True, populate the database with some defaults.
-        enable: A list of component classes or name globs to activate in the
-                stub environment.
+        :param default_data: If True, populate the database with some
+                             defaults.
+        :param enable: A list of component classes or name globs to
+                       activate in the stub environment.
         """
         ComponentManager.__init__(self)
         Component.__init__(self)
@@ -273,6 +280,7 @@ class EnvironmentStub(Environment):
         # -- database
         self.dburi = get_dburi()
         if self.dburi.startswith('sqlite'):
+            self.config.set('trac', 'database', 'sqlite::memory:')
             self.db = InMemoryDatabase()
 
         if default_data:
@@ -284,7 +292,7 @@ class EnvironmentStub(Environment):
 
         self.known_users = []
         translation.activate(Locale and Locale('en', 'US'))
-
+        
     def get_read_db(self):
         return self.get_db_cnx()
     
@@ -382,6 +390,13 @@ class EnvironmentStub(Environment):
         except Exception:
             db.rollback()
 
+    # overriden
+
+    def is_component_enabled(self, cls):
+        if self._component_name(cls).startswith('__main__.'):
+            return True
+        return Environment.is_component_enabled(self, cls)
+
     def get_known_users(self, cnx=None):
         return self.known_users
 
