@@ -29,11 +29,11 @@ from trac.mimeview import Context
 from trac.perm import IPermissionRequestor
 from trac.resource import Resource, ResourceNotFound
 from trac.ticket.api import TicketSystem
-from trac.util import as_int
+from trac.util import as_int, content_disposition
 from trac.util.datefmt import format_datetime, format_time, from_utimestamp
 from trac.util.presentation import Paginator
-from trac.util.text import to_unicode
-from trac.util.translation import _
+from trac.util.text import to_unicode, quote_query_string
+from trac.util.translation import _, tag_
 from trac.web.api import IRequestHandler, RequestDone
 from trac.web.chrome import add_ctxtnav, add_link, add_notice, add_script, \
                             add_stylesheet, add_warning, \
@@ -283,6 +283,9 @@ class ReportModule(Component):
 
         return 'report_list.html', data, None
 
+    _html_cols = set(['__style__', '__color__', '__fgcolor__',
+                         '__bgcolor__', '__grouplink__'])
+
     def _render_view(self, req, id):
         """Retrieve the report results and pre-process them for rendering."""
         db = self.env.get_db_cnx()
@@ -321,7 +324,7 @@ class ReportModule(Component):
                 if query[-1] != '?':
                     query += '&'
                 query += report_id
-            req.redirect(req.href.query() + query)
+            req.redirect(req.href.query() + quote_query_string(query))
         elif query.startswith('query:'):
             try:
                 from trac.ticket.query import Query, QuerySyntaxError
@@ -384,8 +387,8 @@ class ReportModule(Component):
 
         except Exception, e:
             db.rollback()
-            data['message'] = _('Report execution failed: %(error)s',
-                                error=to_unicode(e))
+            data['message'] = tag_('Report execution failed: %(error)s',
+                                   error=tag.pre(to_unicode(e)))
             return 'report_view.html', data, None
 
         paginator = None
@@ -477,14 +480,17 @@ class ReportModule(Component):
         # Structure the rows and cells:
         #  - group rows according to __group__ value, if defined
         #  - group cells the same way headers are grouped
+        chrome = Chrome(self.env)
         row_groups = []
-        authorized_results = [] 
+        authorized_results = []
         prev_group_value = None
         for row_idx, result in enumerate(results):
             col_idx = 0
             cell_groups = []
             row = {'cell_groups': cell_groups}
             realm = 'ticket'
+            parent_realm = ''
+            parent_id = ''
             email_cells = []
             for header_group in header_groups:
                 cell_group = []
@@ -498,12 +504,10 @@ class ReportModule(Component):
                         prev_group_value = value
                         # Brute force handling of email in group by header
                         row_groups.append(
-                            (Chrome(self.env).format_author(req, value), []) )
+                            (value and chrome.format_author(req, value), []))
                     # Other row properties
                     row['__idx__'] = row_idx
-                    if col in ('__style__', '__color__',
-                               '__fgcolor__', '__bgcolor__',
-                               '__grouplink__'):
+                    if col in self._html_cols:
                         row[col] = value
                     if col in ('report', 'ticket', 'id', '_id'):
                         row['id'] = value
@@ -513,17 +517,25 @@ class ReportModule(Component):
                         email_cells.append(cell)
                     elif col == 'realm':
                         realm = value
+                    elif col == 'parent_realm':
+                        parent_realm = value
+                    elif col == 'parent_id':
+                        parent_id = value
                     cell_group.append(cell)
                 cell_groups.append(cell_group)
-            resource = Resource(realm, row.get('id'))
+            if parent_realm:
+                resource = Resource(realm, row.get('id'),
+                                    parent=Resource(parent_realm, parent_id))
+            else:
+                resource = Resource(realm, row.get('id'))
             # FIXME: for now, we still need to hardcode the realm in the action
             if resource.realm.upper()+'_VIEW' not in req.perm(resource):
                 continue
             authorized_results.append(result)
             if email_cells:
                 for cell in email_cells:
-                    emails = Chrome(self.env).format_emails(context(resource),
-                                                            cell['value'])
+                    emails = chrome.format_emails(context(resource),
+                                                  cell['value'])
                     result[cell['index']] = cell['value'] = emails
             row['resource'] = resource
             if row_groups:
@@ -533,14 +545,13 @@ class ReportModule(Component):
                 row_groups = [(None, row_group)]
             row_group.append(row)
 
-
         data.update({'header_groups': header_groups,
                      'row_groups': row_groups,
                      'numrows': numrows,
-                     'sorting_enabled': len(row_groups) == 1})
+                     'sorting_enabled': '__group__' not in cols})
 
         if format == 'rss':
-            data['email_map'] = Chrome(self.env).get_email_map()
+            data['email_map'] = chrome.get_email_map()
             data['context'] = Context.from_request(req, report_resource,
                                                    absurls=True)
             return 'report.rss', data, 'application/rss+xml'
@@ -627,9 +638,9 @@ class ReportModule(Component):
             sort_col = req.args.get('sort', '')
             self.log.debug("Columns %r, Sort column %s" % (cols, sort_col))
             order_cols = []
+            if '__group__' in cols:
+                sort_col = '' # sorting is disabled (#15030)
             if sort_col:
-                if '__group__' in cols:
-                    order_cols.append('__group__')
                 if sort_col in cols:
                     order_cols.append(sort_col)
                 else:
@@ -740,19 +751,20 @@ class ReportModule(Component):
 
         out = StringIO()
         writer = csv.writer(out, delimiter=sep)
-        writer.writerow([unicode(c).encode('utf-8') for c in cols])
+        writer.writerow([unicode(c).encode('utf-8') for c in cols
+                         if c not in self._html_cols])
         for row in rows:
-            row = list(row)
-            for i in xrange(len(row)):
-                row[i] = converters[i](row[i]).encode('utf-8')
-            writer.writerow(row)
+            writer.writerow([converters[i](cell).encode('utf-8')
+                             for i, cell in enumerate(row)
+                             if cols[i] not in self._html_cols])
         data = out.getvalue()
 
         req.send_response(200)
         req.send_header('Content-Type', mimetype + ';charset=utf-8')
         req.send_header('Content-Length', len(data))
         if filename:
-            req.send_header('Content-Disposition', 'filename=' + filename)
+            req.send_header('Content-Disposition',
+                            content_disposition(filename=filename))
         req.end_headers()
         req.write(data)
         raise RequestDone
@@ -773,7 +785,7 @@ class ReportModule(Component):
         req.send_header('Content-Length', len(data))
         if id:
             req.send_header('Content-Disposition',
-                            'filename=report_%s.sql' % id)
+                            content_disposition(filename='report_%s.sql' % id))
         req.end_headers()
         req.write(data)
         raise RequestDone
