@@ -123,15 +123,15 @@ try:
             self._null_translations = NullTranslationsBabel()
             self._plugin_domains = {}
             self._plugin_domains_lock = threading.RLock()
+            self._activate_failed = False
 
         # Public API
 
         def add_domain(self, domain, env_path, locales_dir):
             self._plugin_domains_lock.acquire()
             try:
-                if env_path not in self._plugin_domains:
-                    self._plugin_domains[env_path] = []
-                self._plugin_domains[env_path].append((domain, locales_dir))
+                domains = self._plugin_domains.setdefault(env_path, {})
+                domains[domain] = locales_dir
             finally:
                 self._plugin_domains_lock.release()
 
@@ -142,6 +142,7 @@ try:
             try:
                 locale_dir = pkg_resources.resource_filename('trac', 'locale')
             except Exception:
+                self._activate_failed = True
                 return
             t = Translations.load(locale_dir, locale or 'en_US')
             if not t or t.__class__ is NullTranslations:
@@ -149,12 +150,13 @@ try:
             elif env_path:
                 self._plugin_domains_lock.acquire()
                 try:
-                    domains = list(self._plugin_domains.get(env_path, []))
+                    domains = self._plugin_domains.get(env_path, {}).items()
                 finally:
                     self._plugin_domains_lock.release()
                 for domain, dirname in domains:
                     t.add(Translations.load(dirname, locale, domain))
             self._current.translations = t
+            self._activate_failed = False
          
         def deactivate(self):
             self._current.args = None
@@ -175,7 +177,10 @@ try:
                 get_locale, env_path = self._current.args
                 self._current.args = None
                 self.activate(get_locale(), env_path)
-            return self._current.translations is not None
+            # FIXME: The following always returns True: either a translation is
+            # active, or activation has failed.
+            return self._current.translations is not None \
+                   or self._activate_failed
 
         # Delegated methods
 
