@@ -17,7 +17,7 @@
 import os.path
 import time
 
-from trac.admin import AdminCommandError, IAdminCommandProvider
+from trac.admin import AdminCommandError, IAdminCommandProvider, get_dir_list
 from trac.config import ListOption, Option
 from trac.core import *
 from trac.resource import IResourceManager, Resource, ResourceNotFound
@@ -295,7 +295,7 @@ class RepositoryManager(Component):
         (''since 0.11'').
         
         This option is deprecated, and repositories should be defined in the
-        [[TracIni#repositories-section repositories]] section, or using the
+        [TracIni#repositories-section repositories] section, or using the
         "Repositories" admin panel. (''since 0.12'')""")
 
     repository_sync_per_request = ListOption('trac',
@@ -330,6 +330,11 @@ class RepositoryManager(Component):
                     repo = self.get_repository(reponame)
                     if repo:
                         repo.sync()
+                    else:
+                        self.log.warning("Unable to find repository '%s' for "
+                                         "synchronization",
+                                         reponame or '(default)')
+                        continue
                 except TracError, e:
                     add_warning(req,
                         _("Can't synchronize with repository \"%(name)s\" "
@@ -337,7 +342,7 @@ class RepositoryManager(Component):
                           "information.", name=reponame or '(default)',
                           error=to_unicode(e.message)))
                 self.log.info("Synchronized '%s' repository in %0.2f seconds",
-                              reponame, time.time() - start)
+                              reponame or '(default)', time.time() - start)
         return handler
 
     def post_process_request(self, req, template, data, content_type):
@@ -352,13 +357,17 @@ class RepositoryManager(Component):
 
     def get_resource_description(self, resource, format=None, **kwargs):
         if resource.realm == 'changeset':
-            reponame, id = resource.parent.id, resource.id
+            parent = resource.parent
+            reponame = parent and parent.id
+            id = resource.id
             if reponame:
                 return _("Changeset %(rev)s in %(repo)s", rev=id, repo=reponame)
             else:
                 return _("Changeset %(rev)s", rev=id)
         elif resource.realm == 'source':
-            reponame, id = resource.parent.id, resource.id
+            parent = resource.parent
+            reponame = parent and parent.id
+            id = resource.id
             version = ''
             if format == 'summary':
                 repos = self.get_repository(reponame)
@@ -382,11 +391,14 @@ class RepositoryManager(Component):
 
     def get_resource_url(self, resource, href, **kwargs):
         if resource.realm == 'changeset':
-            return href.changeset(resource.id, resource.parent.id or None)
+            parent = resource.parent
+            return href.changeset(resource.id, parent and parent.id or None)
         elif resource.realm == 'source':
-            return href.source(resource.parent.id or None, resource.id)
+            parent = resource.parent
+            return href.browser(parent and parent.id or None, resource.id,
+                                rev=resource.version or None)
         elif resource.realm == 'repository':
-            return href.source(resource.id or None)
+            return href.browser(resource.id or None)
 
     def resource_exists(self, resource):
         if resource.realm == 'repository':
@@ -713,6 +725,8 @@ class NoSuchNode(ResourceNotFound):
 class Repository(object):
     """Base class for a repository provided by a version control system."""
 
+    scope = '/'
+    
     def __init__(self, name, params, log):
         """Initialize a repository.
         
@@ -839,12 +853,12 @@ class Repository(object):
     def get_oldest_rev(self):
         """Return the oldest revision stored in the repository."""
         raise NotImplementedError
-    oldest_rev = property(lambda x: x.get_oldest_rev())
+    oldest_rev = property(lambda self: self.get_oldest_rev())
 
     def get_youngest_rev(self):
         """Return the youngest revision in the repository."""
         raise NotImplementedError
-    youngest_rev = property(lambda x: x.get_youngest_rev())
+    youngest_rev = property(lambda self: self.get_youngest_rev())
 
     def previous_rev(self, rev, path=''):
         """Return the revision immediately preceding the specified revision."""
@@ -938,8 +952,8 @@ class Node(object):
     DIRECTORY = "dir"
     FILE = "file"
 
-    resource = property(lambda self: Resource('source', self.created_path,
-                                              version=self.created_rev,
+    resource = property(lambda self: Resource('source', self.path,
+                                              version=self.rev,
                                               parent=self.repos.resource))
 
     # created_path and created_rev properties refer to the Node "creation"
@@ -1022,7 +1036,7 @@ class Node(object):
         Will be `None` for a directory.
         """
         raise NotImplementedError
-    content_length = property(lambda x: x.get_content_length())
+    content_length = property(lambda self: self.get_content_length())
 
     def get_content_type(self):
         """The MIME type corresponding to the content, if known.
@@ -1030,18 +1044,18 @@ class Node(object):
         Will be `None` for a directory.
         """
         raise NotImplementedError
-    content_type = property(lambda x: x.get_content_type())
+    content_type = property(lambda self: self.get_content_type())
 
     def get_name(self):
         return self.path.split('/')[-1]
-    name = property(lambda x: x.get_name())
+    name = property(lambda self: self.get_name())
 
     def get_last_modified(self):
         raise NotImplementedError
-    last_modified = property(lambda x: x.get_last_modified())
+    last_modified = property(lambda self: self.get_last_modified())
 
-    isdir = property(lambda x: x.kind == Node.DIRECTORY)
-    isfile = property(lambda x: x.kind == Node.FILE)
+    isdir = property(lambda self: self.kind == Node.DIRECTORY)
+    isfile = property(lambda self: self.kind == Node.FILE)
 
     def can_view(self, perm):
         """Return True if view permission is granted on the node."""
