@@ -72,14 +72,15 @@ def _import_svn():
     Pool.apr_pool_clear = staticmethod(core.apr_pool_clear)
     Pool.apr_pool_destroy = staticmethod(core.apr_pool_destroy)
 
-def _to_svn(*args):
-    """Expect a list of `unicode` path components.
+def _to_svn(pool, *args):
+    """Expect a pool and a list of `unicode` path components.
     
     Returns an UTF-8 encoded string suitable for the Subversion python 
     bindings (the returned path never starts with a leading "/")
     """
-    return '/'.join([p for p in [p.strip('/') for p in args] if p]) \
-           .encode('utf-8')
+    return core.svn_path_canonicalize('/'.join(args).lstrip('/')
+                                                    .encode('utf-8'),
+                                      pool)
 
 def _from_svn(path):
     """Expect an UTF-8 encoded string and transform it to an `unicode` object
@@ -238,6 +239,7 @@ class SvnCachedRepository(CachedRepository):
     def rev_db(self, rev):
         return int(rev or 0)
 
+
 class SubversionConnector(Component):
 
     implements(ISystemInfoProvider, IRepositoryConnector)
@@ -371,7 +373,8 @@ class SubversionRepository(Repository):
             pool = self.pool
         rev = self.normalize_rev(rev)
         rev_root = fs.revision_root(self.fs_ptr, rev, pool())
-        node_type = fs.check_path(rev_root, _to_svn(self.scope, path), pool())
+        node_type = fs.check_path(rev_root, _to_svn(pool(), self.scope, path),
+                                  pool())
         return node_type in _kindmap
 
     def normalize_path(self, path):
@@ -470,7 +473,7 @@ class SubversionRepository(Repository):
         object.
         Must start with `(path, created rev)`.
         """
-        path_utf8 = _to_svn(self.scope, path)
+        path_utf8 = _to_svn(pool(), self.scope, path)
         if start < end:
             start, end = end, start
         if (start, end) == (1, 0): # only happens for empty repos
@@ -632,9 +635,9 @@ class SubversionRepository(Repository):
             text_deltas = 0 # as this is anyway re-done in Diff.py...
             entry_props = 0 # "... typically used only for working copy updates"
             repos.svn_repos_dir_delta(old_root,
-                                      _to_svn(self.scope + old_path), '',
-                                      new_root,
-                                      _to_svn(self.scope + new_path),
+                                      _to_svn(subpool(), self.scope, old_path),
+                                      '', new_root,
+                                      _to_svn(subpool(), self.scope, new_path),
                                       e_ptr, e_baton, authz_cb,
                                       text_deltas,
                                       1, # directory
@@ -652,15 +655,18 @@ class SubversionRepository(Repository):
                                              new_rev)
                 else:
                     kind = _kindmap[fs.check_path(old_root,
-                                                  _to_svn(self.scope,
+                                                  _to_svn(subpool(),
+                                                          self.scope,
                                                           old_node.path),
                                                   subpool())]
                 yield  (old_node, new_node, kind, change)
         else:
             old_root = fs.revision_root(self.fs_ptr, old_rev, subpool())
             new_root = fs.revision_root(self.fs_ptr, new_rev, subpool())
-            if fs.contents_changed(old_root, _to_svn(self.scope, old_path),
-                                   new_root, _to_svn(self.scope, new_path),
+            if fs.contents_changed(old_root,
+                                   _to_svn(subpool(), self.scope, old_path),
+                                   new_root,
+                                   _to_svn(subpool(), self.scope, new_path),
                                    subpool()):
                 yield (old_node, new_node, Node.FILE, Changeset.EDIT)
 
@@ -670,15 +676,14 @@ class SubversionNode(Node):
     def __init__(self, path, rev, repos, pool=None, parent_root=None):
         self.fs_ptr = repos.fs_ptr
         self.scope = repos.scope
-        self._scoped_path_utf8 = _to_svn(self.scope, path)
         self.pool = Pool(pool)
-        self._requested_rev = rev
         pool = self.pool()
+        self._scoped_path_utf8 = _to_svn(pool, self.scope, path)
 
         if parent_root:
             self.root = parent_root
         else:
-            self.root = fs.revision_root(self.fs_ptr, rev, self.pool())
+            self.root = fs.revision_root(self.fs_ptr, rev, pool)
         node_type = fs.check_path(self.root, self._scoped_path_utf8, pool)
         if not node_type in _kindmap:
             raise NoSuchNode(path, rev)
@@ -697,9 +702,8 @@ class SubversionNode(Node):
             self.created_path = _path_within_scope(self.scope, cp)
         else:
             self.created_rev, self.created_path = rev, path
-        self.rev = self.created_rev
         # TODO: check node id
-        Node.__init__(self, repos, path, self.rev, _kindmap[node_type])
+        Node.__init__(self, repos, path, rev, _kindmap[node_type])
 
     def get_content(self):
         if self.isdir:
@@ -718,16 +722,15 @@ class SubversionNode(Node):
         entries = fs.dir_entries(self.root, self._scoped_path_utf8, pool())
         for item in entries.keys():
             path = posixpath.join(self.path, _from_svn(item))
-            yield SubversionNode(path, self._requested_rev, self.repos,
-                                 self.pool, self.root)
+            yield SubversionNode(path, self.rev, self.repos, self.pool,
+                                 self.root)
 
     def get_history(self, limit=None):
         newer = None # 'newer' is the previously seen history tuple
         older = None # 'older' is the currently examined history tuple
         pool = Pool(self.pool)
         numrevs = 0
-        for path, rev in self.repos._history(self.path, 1, self._requested_rev,
-                                             pool):
+        for path, rev in self.repos._history(self.path, 1, self.rev, pool):
             path = _path_within_scope(self.scope, path)
             if rev > 0 and path:
                 older = (path, rev, Changeset.ADD)
@@ -751,7 +754,7 @@ class SubversionNode(Node):
             def blame_receiver(line_no, revision, author, date, line, pool):
                 annotations.append(revision)
             try:
-                rev = _svn_rev(self._requested_rev)
+                rev = _svn_rev(self.rev)
                 start = _svn_rev(0)
                 file_url_utf8 = posixpath.join(self.repos.ra_url_utf8,
                                                self._scoped_path_utf8)
@@ -811,7 +814,7 @@ class SubversionNode(Node):
         operation was performed.
         """
         ancestors = []
-        previous = (self._scoped_path_utf8, self._requested_rev, self.root)
+        previous = (self._scoped_path_utf8, self.rev, self.root)
         while previous:
             (previous_path, previous_rev, previous_root) = previous
             previous = None
@@ -898,18 +901,20 @@ class SubversionChangeset(Changeset):
             path = _from_svn(path_utf8)
             base_path = _from_svn(base_path_utf8)
             base_rev = change.base_rev
+            change_action = getattr(change, 'action', None)
 
             # Ensure `base_path` is within the scope
             if not _is_path_within_scope(self.scope, base_path):
                 base_path, base_rev = None, -1
 
             # Determine the action
-            if not path:                # deletion
-                if new_path:
+            if not path and not new_path and self.scope == '/':
+                action = Changeset.EDIT # root property change
+            elif not path or (change_action is not None
+                              and change_action == repos.CHANGE_ACTION_DELETE):
+                if new_path:            # deletion
                     action = Changeset.DELETE
                     deletions[new_path.lstrip('/')] = idx
-                elif self.scope == '/': # root property change
-                    action = Changeset.EDIT
                 else:                   # deletion outside of scope, ignore
                     continue
             elif change.added or not base_path: # add or copy
