@@ -17,19 +17,18 @@
 
 from datetime import datetime, timedelta
 from fnmatch import fnmatchcase
-import inspect
 import re
 
 from genshi.builder import tag
 
-from trac.config import ListOption, BoolOption, Option, _TRUE_VALUES
+from trac.config import ListOption, BoolOption, Option
 from trac.core import *
 from trac.mimeview.api import Mimeview, is_binary, \
                               IHTMLPreviewAnnotator, Context
 from trac.perm import IPermissionRequestor
 from trac.resource import Resource, ResourceNotFound
 from trac.util import as_bool, embedded_numbers
-from trac.util.compat import any
+from trac.util.compat import any, cleandoc
 from trac.util.datefmt import http_date, to_datetime, utc
 from trac.util.html import escape, Markup
 from trac.util.text import exception_to_unicode, shorten_line
@@ -342,8 +341,9 @@ class BrowserModule(Component):
         reponame, repos, path = rm.get_repository_by_path(path)
 
         # Repository index
-        if not reponame and path == '/':
-            if repos and (all_repositories[''].get('hidden') in _TRUE_VALUES
+        show_index = not reponame and path == '/'
+        if show_index:
+            if repos and (as_bool(all_repositories[''].get('hidden'))
                           or not repos.can_view(req.perm)):
                 repos = None
 
@@ -382,7 +382,7 @@ class BrowserModule(Component):
                                     order, desc)
 
         repo_data = dir_data = file_data = None
-        if not reponame and path == '/':
+        if show_index:
             repo_data = self._render_repository_index(
                                         context, all_repositories, order, desc)
         if node:
@@ -426,7 +426,7 @@ class BrowserModule(Component):
         # Links for contextual navigation
         if node:
             if node.isfile:
-                prev_rev = repos.previous_rev(rev=node.rev,
+                prev_rev = repos.previous_rev(rev=node.created_rev,
                                               path=node.created_path)
                 if prev_rev:
                     href = req.href.browser(reponame,
@@ -436,7 +436,7 @@ class BrowserModule(Component):
                 if rev is not None:
                     add_link(req, 'up', req.href.browser(reponame,
                                                          node.created_path))
-                next_rev = repos.next_rev(rev=node.rev,
+                next_rev = repos.next_rev(rev=node.created_rev,
                                           path=node.created_path)
                 if next_rev:
                     href = req.href.browser(reponame, node.created_path,
@@ -450,23 +450,24 @@ class BrowserModule(Component):
                     add_link(req, 'up', path_links[-2]['href'],
                              _('Parent directory'))
                 add_ctxtnav(req, tag.a(_('Last Change'), 
-                            href=req.href.changeset(node.rev, reponame,
+                            href=req.href.changeset(node.created_rev, reponame,
                                                     node.created_path)))
             if node.isfile:
-                if data['file']['annotate']:
+                annotate = data['file']['annotate']
+                if annotate:
                     add_ctxtnav(req, _('Normal'), 
                                 title=_('View file without annotations'), 
                                 href=req.href.browser(reponame,
                                                       node.created_path, 
-                                                      rev=node.rev))
-                else:
-                    add_ctxtnav(req, _('Annotate'), 
+                                                      rev=rev))
+                if annotate != 'blame':
+                    add_ctxtnav(req, _('Blame'), 
                                 title=_('Annotate each line with the last '
                                         'changed revision '
                                         '(this can be time consuming...)'), 
                                 href=req.href.browser(reponame,
                                                       node.created_path, 
-                                                      rev=node.rev,
+                                                      rev=rev,
                                                       annotate='blame'))
             add_ctxtnav(req, _('Revision Log'), 
                         href=req.href.log(reponame, path, rev=rev))
@@ -490,7 +491,7 @@ class BrowserModule(Component):
         rm = RepositoryManager(self.env)
         repositories = []
         for reponame, repoinfo in all_repositories.iteritems():
-            if not reponame or repoinfo.get('hidden') in _TRUE_VALUES:
+            if not reponame or as_bool(repoinfo.get('hidden')):
                 continue
             try:
                 repos = rm.get_repository(reponame)
@@ -541,14 +542,17 @@ class BrowserModule(Component):
 
         # Entries metadata
         class entry(object):
-            __slots__ = 'name rev kind isdir path content_length'.split()
+            __slots__ = ('name', 'rev', 'created_rev', 'kind', 'isdir', 'path',
+                         'content_length')
+
             def __init__(self, node):
                 for f in entry.__slots__:
                     setattr(self, f, getattr(node, f))
                 
         entries = [entry(n) for n in node.get_entries()
                    if n.can_view(req.perm)]
-        changes = get_changes(repos, [i.rev for i in entries], self.log)
+        changes = get_changes(repos, [i.created_rev for i in entries],
+                              self.log)
 
         if rev:
             newest = repos.get_changeset(rev).date
@@ -571,7 +575,7 @@ class BrowserModule(Component):
         # Ordering of entries
         if order == 'date':
             def file_order(a):
-                return (changes[a.rev].date,
+                return (changes[a.created_rev].date,
                         embedded_numbers(a.name.lower()))
         elif order == 'size':
             def file_order(a):
@@ -579,7 +583,7 @@ class BrowserModule(Component):
                         embedded_numbers(a.name.lower()))
         elif order == 'author':
             def file_order(a):
-                return (changes[a.rev].author.lower(),
+                return (changes[a.created_rev].author.lower(),
                         embedded_numbers(a.name.lower()))
         else:
             def file_order(a):
@@ -653,7 +657,7 @@ class BrowserModule(Component):
         else:
             # The changeset corresponding to the last change on `node` 
             # is more interesting than the `rev` changeset.
-            changeset = repos.get_changeset(node.rev)
+            changeset = repos.get_changeset(node.created_rev)
 
             # add ''Plain Text'' alternate link if needed
             if not is_binary(chunk) and mime_type != 'text/plain':
@@ -676,21 +680,20 @@ class BrowserModule(Component):
             add_stylesheet(req, 'common/css/code.css')
 
             annotations = ['lineno']
-            force_source = False
-            if 'annotate' in req.args:
-                force_source = True
-                annotations.insert(0, req.args['annotate'])
+            annotate = req.args.get('annotate')
+            if annotate:
+                annotations.insert(0, annotate)
             preview_data = mimeview.preview_data(context, node.get_content(),
                                                  node.get_content_length(),
                                                  mime_type, node.created_path,
                                                  raw_href,
                                                  annotations=annotations,
-                                                 force_source=force_source)
+                                                 force_source=bool(annotate))
             return {
                 'changeset': changeset,
                 'size': node.content_length,
                 'preview': preview_data,
-                'annotate': force_source,
+                'annotate': annotate,
                 }
 
     # public methods
@@ -794,7 +797,7 @@ class BrowserModule(Component):
         yield "RepositoryIndex"
 
     def get_macro_description(self, name):
-        return inspect.cleandoc("""
+        return cleandoc("""
         Display the list of available repositories.
 
         Can be given the following named arguments:
@@ -853,7 +856,7 @@ class BrowserModule(Component):
                      for reponame in all_repos]
         all_repos = sorted(((reponame, repos) for reponame, repos in all_repos
                             if repos
-                            and repos.params.get('hidden') not in _TRUE_VALUES
+                            and not as_bool(repos.params.get('hidden'))
                             and repos.can_view(formatter.perm)),
                            reverse=desc)
 
