@@ -28,7 +28,7 @@ from trac.perm import IPermissionRequestor
 from trac.resource import ResourceNotFound
 from trac.util import Ranges
 from trac.util.compat import any
-from trac.util.text import wrap
+from trac.util.text import to_unicode, wrap
 from trac.util.translation import _
 from trac.versioncontrol.api import RepositoryManager, Changeset, \
                                     NoSuchChangeset
@@ -126,12 +126,13 @@ class LogModule(Component):
                 ranges = list(revranges.pairs)
                 ranges.reverse()
                 for (a, b) in ranges:
-                    while b >= a:
-                        rev = repos.normalize_rev(b)
-                        node = get_existing_node(req, repos, prevpath, rev)
+                    a = repos.normalize_rev(a)
+                    b = repos.normalize_rev(b)
+                    while not repos.rev_older_than(b, a):
+                        node = get_existing_node(req, repos, prevpath, b)
                         node_history = list(node.get_history(2))
                         p, rev, chg = node_history[0]
-                        if rev < a:
+                        if repos.rev_older_than(rev, a):
                             break # simply skip, no separator
                         if 'CHANGESET_VIEW' in req.perm(cset_resource(id=rev)):
                             if expected_next_item:
@@ -141,7 +142,7 @@ class LogModule(Component):
                                     yield (np, nrev, None)
                             yield node_history[0]
                         prevpath = node_history[-1][0] # follow copy
-                        b = rev - 1
+                        b = repos.previous_rev(rev)
                         if len(node_history) > 1:
                             expected_next_item = node_history[-1]
                         else:
@@ -359,37 +360,41 @@ class LogModule(Component):
                 path, revs = match[:idx], match[idx+1:]
         
         rm = RepositoryManager(self.env)
-        reponame, repos, path = rm.get_repository_by_path(path)
-        if not reponame:
-            reponame = rm.get_default_repository(formatter.context)
-            if reponame is not None:
-                repos = rm.get_repository(reponame)
+        try:
+            reponame, repos, path = rm.get_repository_by_path(path)
+            if not reponame:
+                reponame = rm.get_default_repository(formatter.context)
+                if reponame is not None:
+                    repos = rm.get_repository(reponame)
 
-        if repos:
-            revranges = None
-            if any(c for c in ':-,' if c in revs):
-                revranges = self._normalize_ranges(repos, path, revs)
-                revs = None
-            if 'LOG_VIEW' in formatter.perm:
-                if revranges:
-                    href = formatter.href.log(repos.reponame or None,
-                                              path or '/', revs=str(revranges))
-                else:
-                    try:
-                        rev = repos.normalize_rev(revs)
-                    except NoSuchChangeset:
-                        rev = None
-                    href = formatter.href.log(repos.reponame or None,
-                                              path or '/', rev=rev)
-                if query and (revranges or revs):
-                    query = '&' + query[1:]
-                return tag.a(label, class_='source',
-                             href=href + query + fragment)
-            errmsg = _("No permission to view change log")
-        elif reponame:
-            errmsg = _("Repository '%(repo)s' not found", repo=reponame)
-        else:
-            errmsg = _("No default repository defined")
+            if repos:
+                revranges = None
+                if any(c for c in ':-,' if c in revs):
+                    revranges = self._normalize_ranges(repos, path, revs)
+                    revs = None
+                if 'LOG_VIEW' in formatter.perm:
+                    if revranges:
+                        href = formatter.href.log(repos.reponame or None,
+                                                  path or '/', 
+                                                  revs=str(revranges))
+                    else:
+                        try:
+                            rev = repos.normalize_rev(revs)
+                        except NoSuchChangeset:
+                            rev = None
+                        href = formatter.href.log(repos.reponame or None,
+                                                  path or '/', rev=rev)
+                    if query and (revranges or revs):
+                        query = '&' + query[1:]
+                    return tag.a(label, class_='source',
+                                 href=href + query + fragment)
+                errmsg = _("No permission to view change log")
+            elif reponame:
+                errmsg = _("Repository '%(repo)s' not found", repo=reponame)
+            else:
+                errmsg = _("No default repository defined")
+        except TracError, e:
+            errmsg = to_unicode(e)
         return tag.a(label, class_='missing source', title=errmsg)
 
     LOG_LINK_RE = re.compile(r"([^@:]*)[@:]%s?" % REV_RANGE)
