@@ -116,10 +116,10 @@ def parse_arg_list(query_string):
         else:
             (name, value) = (nv[0], empty)
         name = unquote(name.replace('+', ' '))
-        if isinstance(name, unicode):
-            name = name.encode('utf-8')
+        if isinstance(name, str):
+            name = unicode(name, 'utf-8')
         value = unquote(value.replace('+', ' '))
-        if not isinstance(value, unicode):
+        if isinstance(value, str):
             value = unicode(value, 'utf-8')
         args.append((name, value))
     return args
@@ -408,6 +408,7 @@ class Request(object):
     def send(self, content, content_type='text/html', status=200):
         self.send_response(status)
         self.send_header('Cache-Control', 'must-revalidate')
+        self.send_header('Expires', 'Fri, 01 Jan 1999 00:00:00 GMT')
         self.send_header('Content-Type', content_type + ';charset=utf-8')
         self.send_header('Content-Length', len(content))
         self.end_headers()
