@@ -679,9 +679,7 @@ class Chrome(Component):
         req.hdf['chrome.logo'] = req.chrome['logo']
 
         for category, items in req.chrome['nav'].items():
-            for item in items:
-                prefix = 'chrome.nav.%s.%s' % (category, item['name'])
-                req.hdf[prefix] = item['label']
+            req.hdf['chrome.nav.%s' % category] = items
 
     def populate_data(self, req, data):
         d = self._default_context_data.copy()
@@ -844,7 +842,7 @@ class Chrome(Component):
 
         if method == 'text':
             buffer = StringIO()
-            stream.render('text', out=buffer)
+            stream.render('text', out=buffer, encoding='utf-8')
             return buffer.getvalue()
 
         doctype = {'text/html': DocType.XHTML_STRICT}.get(content_type)
@@ -856,22 +854,27 @@ class Chrome(Component):
 
         links = req.chrome.get('links')
         scripts = req.chrome.get('scripts')
+        script_data = req.chrome.get('script_data')
         req.chrome['links'] = {}
         req.chrome['scripts'] = []
+        req.chrome['script_data'] = {}
         data.setdefault('chrome', {}).update({
             'late_links': req.chrome['links'],
             'late_scripts': req.chrome['scripts'],
+            'late_script_data': req.chrome['script_data'],
         })
 
         try:
             buffer = StringIO()
-            stream.render(method, doctype=doctype, out=buffer)
+            stream.render(method, doctype=doctype, out=buffer,
+                          encoding='utf-8')
             return buffer.getvalue().translate(_translate_nop,
                                                _invalid_control_chars)
         except Exception, e:
             # restore what may be needed by the error template
             req.chrome['links'] = links
             req.chrome['scripts'] = scripts
+            req.chrome['script_data'] = script_data
             # give some hints when hitting a Genshi unicode error
             if isinstance(e, UnicodeError):
                 pos = self._stream_location(stream)
