@@ -139,11 +139,20 @@ class DetachedSession(dict):
                     WHERE sid=%s AND authenticated=%s
                     """, (self.sid, authenticated))
                 self._old = dict(self.items())
-                cursor.executemany("""
-                    INSERT INTO session_attribute
-                      (sid,authenticated,name,value)
-                    VALUES (%s,%s,%s,%s)
-                    """, [(self.sid, authenticated, k, v) for k, v in items])
+                # The session variables might already have been updated by a
+                # concurrent request.
+                try:
+                    cursor.executemany("""
+                        INSERT INTO session_attribute
+                          (sid,authenticated,name,value)
+                        VALUES (%s,%s,%s,%s)
+                        """, [(self.sid, authenticated, k, v)
+                              for k, v in items])
+                except Exception:
+                    self.env.log.warning('Attributes for session %s already '
+                                         'updated', self.sid)
+                    db.rollback()
+                    return
                 session_saved[0] = True
 
         # Purge expired sessions. We do this only when the session was
@@ -292,11 +301,18 @@ class Session(DetachedSession):
                         WHERE sid=%s
                         """, (self.req.authname, sid))
             else:
-                # we didn't have an anonymous session for this sid
-                cursor.execute("""
-                    INSERT INTO session (sid,last_visit,authenticated)
-                    VALUES (%s,%s,1)
-                    """, (self.req.authname, int(time.time())))
+                # We didn't have an anonymous session for this sid. The
+                # authenticated session might have been inserted between the
+                # SELECT above and here, so we catch the error.
+                try:
+                    cursor.execute("""
+                        INSERT INTO session (sid,last_visit,authenticated)
+                        VALUES (%s,%s,1)
+                        """, (self.req.authname, int(time.time())))
+                except Exception:
+                    self.env.log.warning('Authenticated session for %s '
+                                         'already exists', self.req.authname)
+                    db.rollback()
         self._new = False
 
         self.sid = sid
@@ -495,6 +511,6 @@ class SessionAdmin(Component):
             cursor.execute("""
                 DELETE FROM session_attribute
                 WHERE authenticated=0
-                      AND sid IN (SELECT sid FROM session
-                                  WHERE authenticated=0 AND last_visit<%s)
-                """, (ts,))
+                      AND sid NOT IN (SELECT sid FROM session
+                                      WHERE authenticated=0)
+                """)
