@@ -20,7 +20,7 @@ import os
 import re
 from StringIO import StringIO
 
-from genshi.builder import Element, tag
+from genshi.builder import tag
 from genshi.core import Markup
 
 from trac.core import *
@@ -28,9 +28,9 @@ from trac.resource import Resource, ResourceNotFound, get_resource_name, \
                           get_resource_summary, get_resource_url
 from trac.util.compat import any, rpartition
 from trac.util.datefmt import format_date, from_utimestamp
-from trac.util.html import escape
+from trac.util.html import escape, find_element
 from trac.util.presentation import separated
-from trac.util.text import unquote, to_unicode
+from trac.util.text import unicode_quote, to_unicode
 from trac.util.translation import _
 from trac.wiki.api import IWikiMacroProvider, WikiSystem, parse_args
 from trac.wiki.formatter import format_to_html, format_to_oneliner, \
@@ -91,13 +91,15 @@ class TitleIndexMacro(WikiMacroBase):
        all pages in the hierarchy will be shown.
     """
 
-    SPLIT_RE = re.compile(r"([/ 0-9.]+)")
+    SPLIT_RE = re.compile(r"(/| )")
+    NUM_SPLIT_RE = re.compile(r"([0-9.]+)")
 
     def expand_macro(self, formatter, name, content):
         args, kw = parse_args(content)
         prefix = args and args[0].strip() or None
         hideprefix = args and len(args) > 1 and args[1].strip() == 'hideprefix'
-        minsize = max(int(kw.get('min', 2)), 2)
+        minsize = max(int(kw.get('min', 1)), 1)
+        minsize_group = max(minsize, 2)
         depth = int(kw.get('depth', -1))
         start = prefix and prefix.count('/') or 0
         format = kw.get('format', '')
@@ -129,8 +131,9 @@ class TitleIndexMacro(WikiMacroBase):
             """
             page_paths = []
             for page in pages:
-                path = [elt.rstrip('/').strip() for elt in self.SPLIT_RE.split(
-                        wiki.format_page_name(omitprefix(page), split=True))]
+                path = [elt.strip() for elt in self.SPLIT_RE.split(
+                        self.NUM_SPLIT_RE.sub(r" \1 ", 
+                        wiki.format_page_name(omitprefix(page), split=True)))]
                 page_paths.append(([elt for elt in path if elt], page))
             return page_paths
 
@@ -142,8 +145,8 @@ class TitleIndexMacro(WikiMacroBase):
             return [(wiki.format_page_name(omitprefix(page)).split("/"), page)
                     for page in pages]
 
-        # create the group hierarchy (same for group and hierarchy formats)
-        def split_in_groups(entries):
+        # the different tree structures, each corresponding to its rendering
+        def tree_group(entries):
             """Transform a flat list of entries into a tree structure.
             
             `entries` is a list of `(path_elements, page_name)` pairs
@@ -163,44 +166,95 @@ class TitleIndexMacro(WikiMacroBase):
                 # grouping
                 grouped_entries = [(path_elements[1:], page_name)
                                    for path_elements, page_name in grouper]
-                if key and len(grouped_entries) >= minsize:
-                    subnodes = split_in_groups(sorted(grouped_entries))
+
+                if key and len(grouped_entries) >= minsize_group:
+                    subnodes = tree_group(sorted(grouped_entries))
                     if len(subnodes) == 1:
                         subkey, subnodes = subnodes[0]
-                        node = (key + subkey, subnodes) # FIXME
+                        node = (key + subkey, subnodes)
+                        groups.append(node)
+                    elif self.SPLIT_RE.match(key):
+                        for elt in subnodes:
+                            if isinstance(elt, tuple):
+                                subkey, subnodes = elt
+                                elt = (key + subkey, subnodes)
+                            groups.append(elt)
                     else:
                         node = (key, subnodes)
-                    groups.append(node)
+                        groups.append(node)                    
                 else:
                     for path_elements, page_name in grouped_entries:
                         groups.append(page_name)
             return groups
 
+        def tree_hierarchy(entries):
+            """Transform a flat list of entries into a tree structure.
+            
+            `entries` is a list of `(path_elements, page_name)` pairs
+            
+            Return a list organized in a tree structure, in which:
+              - a leaf is a `(rest, page)` pair, where:
+                - `rest` is the rest of the path to be shown
+                - `page` is a page name
+              - a node is a `(key, nodes, page)` pair, where:
+                - `key` is the leftmost of the path elements, common to the
+                  grouped (path element, page_name) entries
+                - `page` is a page name (if one exists for that node)
+                - `nodes` is a list of nodes or leaves
+            """
+            groups = []
+
+            for key, grouper in groupby(entries, lambda (elts, name):
+                                                    elts and elts[0] or ''):
+                grouped_entries  = [e for e in grouper]
+                sub_entries  = [e for e in grouped_entries if len(e[0]) > 1]
+                key_entries  = [e for e in grouped_entries if len(e[0]) == 1]
+                key_entry = key_entries and key_entries[0] or None
+                key_page = key_entries and key_entry[1] or None
+
+                if key and len(sub_entries) >= minsize:
+                    # remove key from path_elements in grouped entries for
+                    # further grouping
+                    sub_entries = [(path_elements[1:], page)
+                                   for path_elements, page in sub_entries]
+
+                    subnodes = tree_hierarchy(sorted(sub_entries))
+                    node = (key, key_page, subnodes)
+                    groups.append(node)
+                else:
+                    if key_entry:
+                        groups.append(key_entry)
+                    groups.extend(sub_entries)
+            return groups
+            
         # the different rendering formats
         def render_group(group):
             return tag.ul(
-                tag.li(isinstance(elt, tuple) and 
-                       tag(tag.strong(elt[0]), render_group(elt[1])) or
-                       tag.a(wiki.format_page_name(elt),
+                tag.li(isinstance(elt, tuple) and
+                       tag(tag.strong(elt[0].strip('/')), render_group(elt[1]))
+                       or tag.a(wiki.format_page_name(omitprefix(elt)),
                              href=formatter.href.wiki(elt)))
                 for elt in group)
 
         def render_hierarchy(group):
             return tag.ul(
-                tag.li(isinstance(elt, tuple) and 
-                       tag(tag.a(elt[0], href=formatter.href.wiki(elt[0])),
-                           render_hierarchy(elt[1][0:])) or
-                       tag.a(rpartition(elt, '/')[2],
-                             href=formatter.href.wiki(elt)))
+                tag.li(len(elt) == 3 and tag(elt[1] and 
+                           tag.a(elt[0], href=formatter.href.wiki(elt[1]))
+                           or tag(elt[0]),
+                           render_hierarchy(elt[2]))
+                       or
+                       tag.a('/'.join(elt[0]),
+                             href=formatter.href.wiki(elt[1])))
                 for elt in group)
         
-        splitter, renderer = {
-            'group':     (split_pages_group,     render_group),
-            'hierarchy': (split_pages_hierarchy, render_hierarchy),
-            }.get(format, (None, None))
+        transform = {
+            'group': lambda p: render_group(tree_group(split_pages_group(p))),
+            'hierarchy': lambda p: render_hierarchy(
+                                    tree_hierarchy(split_pages_hierarchy(p))),
+            }.get(format, None)
 
-        if splitter and renderer:
-            titleindex = renderer(split_in_groups(splitter(pages)))
+        if transform:
+            titleindex = transform(pages)
         else:
             titleindex = tag.ul(
                 tag.li(tag.a(wiki.format_page_name(omitprefix(page)),
@@ -411,6 +465,16 @@ class ImageMacro(WikiMacroBase):
         attr = {}
         style = {}
         link = ''
+        # helper for the special case `source:`
+        #
+        from trac.versioncontrol.web_ui import BrowserModule
+        # FIXME: somehow use ResourceSystem.get_known_realms()
+        #        ... or directly trac.wiki.extract_link
+        try:
+            browser_links = [res[0] for res in
+                             BrowserModule(self.env).get_link_resolvers()]
+        except Exception:
+            browser_links = []
         while args:
             arg = args.pop(0).strip()
             if size_re.match(arg):
@@ -421,8 +485,9 @@ class ImageMacro(WikiMacroBase):
             elif arg.startswith('link='):
                 val = arg.split('=', 1)[1]
                 elt = extract_link(self.env, formatter.context, val.strip())
+                elt = find_element(elt, 'href')
                 link = None
-                if isinstance(elt, Element):
+                if elt is not None:
                     link = elt.attrib.get('href')
             elif arg in ('left', 'right'):
                 style['float'] = arg
@@ -459,30 +524,36 @@ class ImageMacro(WikiMacroBase):
         url = raw_url = desc = None
         attachment = None
         if (parts and parts[0] in ('http', 'https', 'ftp')): # absolute
-            raw_url = url = desc = filespec
+            raw_url = url = filespec
+            desc = url.rsplit('?')[0]
         elif filespec.startswith('//'):       # server-relative
-            raw_url = url = desc = filespec[1:]
+            raw_url = url = filespec[1:]
+            desc = url.rsplit('?')[0]
         elif filespec.startswith('/'):        # project-relative
-            # use href, but unquote to allow args (use default html escaping)
-            raw_url = url = desc = unquote(formatter.href(filespec))
+            params = ''
+            if '?' in filespec:
+                filespec, params = filespec.rsplit('?', 1)
+            url = formatter.href(filespec)
+            if params:
+                url += '?' + params
+            raw_url, desc = url, filespec
         elif len(parts) == 3:                 # realm:id:attachment-filename
+            #                                 # or intertrac:realm:id 
             realm, id, filename = parts
-            attachment = Resource(realm, id).child('attachment', filename)
+            intertrac_target = "%s:%s" % (id, filename)
+            it = formatter.get_intertrac_url(realm, intertrac_target)
+            if it:
+                url, desc = it
+                raw_url = url + unicode_quote('?format=raw')
+            else:
+                attachment = Resource(realm, id).child('attachment', filename)
         elif len(parts) == 2:
-            # FIXME: somehow use ResourceSystem.get_known_realms()
-            #        ... or directly trac.wiki.extract_link
-            from trac.versioncontrol.web_ui import BrowserModule
-            try:
-                browser_links = [res[0] for res in
-                                 BrowserModule(self.env).get_link_resolvers()]
-            except Exception:
-                browser_links = []
-            if parts[0] in browser_links:   # source:path
+            realm, filename = parts
+            if realm in browser_links:   # source:path
                 # TODO: use context here as well
-                realm, filename = parts
                 rev = None
                 if '@' in filename:
-                    filename, rev = filename.split('@')
+                    filename, rev = filename.rsplit('@', 1)
                 url = formatter.href.browser(filename, rev=rev)
                 raw_url = formatter.href.browser(filename, rev=rev,
                                                  format='raw')
