@@ -43,6 +43,7 @@ from genshi.builder import tag
 from trac.config import BoolOption, Option
 from trac.core import Component, implements
 from trac.perm import PermissionCache
+from trac.resource import Resource
 from trac.ticket import Ticket
 from trac.ticket.notification import TicketNotifyEmail
 from trac.util.compat import any
@@ -131,8 +132,8 @@ class CommitTicketUpdater(Component):
     
     ticket_prefix = '(?:#|(?:ticket|issue|bug)[: ]?)'
     ticket_reference = ticket_prefix + '[0-9]+'
-    ticket_command = ('(?P<action>[A-Za-z]*).?'
-                      '(?P<ticket>%s(?:(?:[, &]*|[ ]?and[ ]?)%s)*)' %
+    ticket_command = (r'(?P<action>[A-Za-z]*)\s*.?\s*'
+                      r'(?P<ticket>%s(?:(?:[, &]*|[ ]?and[ ]?)%s)*)' %
                       (ticket_reference, ticket_reference))
     
     @property
@@ -197,7 +198,7 @@ class CommitTicketUpdater(Component):
         if repos.reponame:
             revstring += '/' + repos.reponame
         return """\
-In [%s]:
+In [changeset:%s]:
 {{{
 #!CommitTicketReference repository="%s" revision="%s"
 %s
@@ -270,24 +271,26 @@ class CommitTicketReferenceMacro(WikiMacroBase):
     """
     
     def expand_macro(self, formatter, name, content, args={}):
-        reponame = args.get('repository')
+        reponame = args.get('repository') or ''
         rev = args.get('revision')
         repos = RepositoryManager(self.env).get_repository(reponame)
-        if repos:
+        try:
             changeset = repos.get_changeset(rev)
             message = changeset.message
             rev = changeset.rev
-        else:
+            resource = repos.resource
+        except Exception:
             message = content
+            resource = Resource('repository', reponame)
         if formatter.context.resource.realm == 'ticket':
             ticket_re = CommitTicketUpdater.ticket_re
-            if not any(int(tkt_id) == formatter.context.resource.id
+            if not any(int(tkt_id) == int(formatter.context.resource.id)
                        for tkt_id in ticket_re.findall(message)):
                 return tag.p("(The changeset message doesn't reference this "
                              "ticket)", class_='hint')
         if ChangesetModule(self.env).wiki_format_messages:
             return tag.div(format_to_html(self.env,
-                formatter.context('changeset', rev, parent=repos.resource),
+                formatter.context('changeset', rev, parent=resource),
                 message, escape_newlines=True), class_='message')
         else:
             return tag.pre(message, class_='message')
